/**
# Example to use srs-librtmp
# see: https://github.com/ossrs/srs/wiki/v2_CN_SrsLibrtmp
    gcc example.c srs_librtmp.cpp -g -O0 -lstdc++ -o example
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
       
#include "srs_librtmp.h"

static off_t file_size;

char* get_data_frame(char* raw_file)
{
    int raw_fd = open(raw_file, O_RDONLY);
    if (raw_fd < 0) {
        srs_human_trace("open h264 raw file %s failed.", raw_file);
        return NULL;
    }
    
    file_size = lseek(raw_fd, 0, SEEK_END);
    if (file_size <= 0) {
        srs_human_trace("h264 raw file %s empty.", raw_file);
        return NULL;
    }
    srs_human_trace("read entirely h264 raw file, size=%dKB", (int)(file_size / 1024));
    
    char*h264_raw = (char*)malloc(file_size);
    if (!h264_raw) {
        srs_human_trace("alloc raw buffer failed for file %s.", raw_file);
        return NULL;
    }
    
    lseek(raw_fd, 0, SEEK_SET);
    ssize_t nb_read = 0;
    if ((nb_read = read(raw_fd, h264_raw, file_size)) != file_size) {
        srs_human_trace("buffer %s failed, expect=%dKB, actual=%dKB.", 
            raw_file, (int)(file_size / 1024), (int)(nb_read / 1024));
        return NULL;
    }

    close(raw_fd);

    return h264_raw; 
}


int main(int argc, char** argv) 
{   
    printf("Example for srs-librtmp\n");
    printf("SRS(ossrs) client librtmp library.\n");
    printf("version: %d.%d.%d\n", srs_version_major(), srs_version_minor(), srs_version_revision());
        
    // connect rtmp context
    const char* rtmp_url = "rtmp://127.0.0.1:1935/live/livestream";
    srs_rtmp_t rtmp = srs_rtmp_create(rtmp_url);
    
    if (srs_rtmp_handshake(rtmp) != 0) {
        srs_human_trace("simple handshake failed.");
        goto rtmp_destroy;
    }
    srs_human_trace("simple handshake success");
    
    if (srs_rtmp_connect_app(rtmp) != 0) {
        srs_human_trace("connect vhost/app failed.");
        goto rtmp_destroy;
    }
    srs_human_trace("connect vhost/app success");
    
    if (srs_rtmp_publish_stream(rtmp) != 0) {
        srs_human_trace("publish stream failed.");
        goto rtmp_destroy;
    }
    srs_human_trace("publish stream success");

    int n = 0;
    int dts = 0;
    int pts = 0;
    double fps = 25;

    while(1){
     
        char str[32];
        sprintf(str,"./h264_nalu_frame/test%d.264",n);  
        char* h264_raw = get_data_frame(str);

        if(h264_raw != NULL){
            // send out the h264 packet over RTMP
            int ret = srs_h264_write_raw_frames(rtmp, h264_raw, file_size, dts, pts);
            if (ret != 0) {
                if (srs_h264_is_dvbsp_error(ret)) {
                    srs_human_trace("ignore drop video error, code=%d", ret);
                } else if (srs_h264_is_duplicated_sps_error(ret)) {
                    srs_human_trace("ignore duplicated sps, code=%d", ret);
                } else if (srs_h264_is_duplicated_pps_error(ret)) {
                    srs_human_trace("ignore duplicated pps, code=%d", ret);
                } else {
                    srs_human_trace("send h264 raw data failed. ret=%d", ret);
                    goto rtmp_destroy;
                }
            }
        }
        else break;

        dts += 1000 / fps;
        pts = dts;
        usleep(40 * 1000);

        free(h264_raw);
        if(++n > 31) n=0;
    }

rtmp_destroy:
    srs_rtmp_destroy(rtmp);
    
    return 0;
}

